<?php
// Auto Answer BOT to derive to service groups. You must configure the 
// $service_groups array with the resonses as key and the name of the 
// service group as value. You might als want to adjust the greetings
// in the $config array messages.
//
// If a channel is handled by the bot it will Greet the user and give
// the option to select any of the service groups defined, when the user
// selects, the chat will be transferred to that service group

$DEBUG=0;

$service_groups = array(
    "A" => "Ventas comerciales",
    "B" => "Soporte técnico",
);

$config = array(
   'initial_greeting'   => 'Gracias por contactarnos!',
   'question_structure' => 'Para contactar con %s, responda "%s"',
   'invalid_answer_msg' => 'No entendí la respuesta.',
   'close_msg'          => 'Gracias!', 
   'transfer_msg'       => 'Gracias, aguarde un momento por favor mientras lo conectamos con el departamento de %s.',
   'last_action'        => 'transfer',
);

// No need to modify below this line

$question_body = $config['initial_greeting']."\n\n";
foreach($service_groups as $key=>$val) {
    $question_body.=sprintf($config['question_structure'],$val,$key)."\n";
}

// Case insensitive key map
$service_groups_tolower=array();
foreach($service_groups as $key=>$val) {
    $service_groups_tolower[strtolower($key)]=$val;
}

$questions = array(
    array(
       "body"  => $question_body,
       "type"  => "transfer",
       "extra" => $service_groups_tolower,
    )
);

$json = file_get_contents('php://input');
$data = json_decode($json, true);

if(!isset($data)) { exit; }

$step = 0;
if(isset($data['state'])) {
    $step = intval($data['state']);
}

if($step > 0) {
    // Validate previous answer
    $prev_answer = $data['body'];
    $validate_result = validate_answer($step-1, $prev_answer);

    if($validate_result == '[wrong]') {
        $msg = retrieve_question($step-1);
        $msg = str_replace($config['initial_greeting'],'',$msg);
        $return = array("body" => $config['invalid_answer_msg']." ".$msg, "attach" => '', "state" => $step);
    } else if ($validate_result == '[close]') {
        $return = array("body" => $config['close_msg'], "attach" => '', "state" => '');
    } else if (preg_match("/\[transfer\^/", $validate_result)) {
        $department = substr($validate_result, 10, -1); 
        $msg = sprintf($config['transfer_msg'], $department)." ".$validate_result;
        $return = array("body" => $msg, "attach" => '', "state" => '');
    } else {
        // Valid answer, get next question
        $msg = retrieve_question($step);
        $step++;
        $return = array("body" => $msg, "attach" => '', "state" => $step); 
    }

} else {
    // Initial question
    $msg = retrieve_question(0);
    $return = array("body" => $msg, "attach" => '', "state" => 1);
}

echo json_encode($return);
exit;

function retrieve_question($n) {
    global $questions, $config;

    if(!isset($questions[$n])) {
        return $config['last_action']; 
    }

    return $questions[$n]['body'];
}

function validate_answer($n, $answer) {
    global $questions, $config;

    if(!isset($questions[$n])) {
        return $config['last_action'];
    }

    $type = $questions[$n]['type'];

    if($type == 'range') {
        $valid = preg_split("/,/", $questions[$n]['extra']);
        if(!in_array($answer, $valid)) {
            return '[wrong]';
        }
    } else if($type == 'transfer') {
        $answer = strtolower($answer);
        $valid = $questions[$n]['extra'];
        if(!array_key_exists($answer, $valid)) {
            return '[wrong]';
        } else {
            return "[transfer^" . $valid[$answer] . "]";
        }
    }

    return '';
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}
